Object.derive = function(prototype, members) {
    var constructor = function() {};
    constructor.prototype = prototype;
    var obj = new constructor();
    Object.extend(obj, members);
    return obj;
}

Object.copy = function() {
    return $A(arguments).inject({}, function(result, obj) {
        return Object.extend(result, obj);
    })
}

Function.prototype.subclass = function(methods) {
    var newClass = Class.create();
    newClass.prototype = Object.derive(this.prototype, methods);
    return newClass;
}

Function.prototype.bindArgs = function() {
    var boundArgs = $A(arguments);
    var func = this;
    return function() {
        return func.apply(this, boundArgs.concat($A(arguments)));
    }
}

Function.prototype.unbind = function() {
    var func = this;
    return function() {
        var args = $A(arguments);
        var target = args.shift();
        return func.apply(target, args);
    }
}

Function.chain = function(funcArray) {
    var next = funcArray[funcArray.length-1];
    for (var i = funcArray.length-2; i >=0; i--) {
        next = funcArray.bindArgs(next);
    }
    return next;
}

Function.prototype.callWithCallbackArgs = function() {
    new CallbackCollector($A(arguments), this);
}

CallbackCollector = Class.create();
CallbackCollector.prototype = {
    initialize: function(funcs, endCallback) {
        this.funcs = funcs;
        this.index= 0;
        this.endCallback = endCallback;
        this.collected = [];
        this.callback();
    },

    callback: function() {
        this.collected = this.collected.concat($A(arguments));
        if (this.index == this.funcs.length) {
            this.endCallback.apply(null, this.collected);
        } else {
            this.funcs[this.index++](this.callback.bind(this));
        }
    }
}

Array.prototype.index = function(keyFunc) {
    var idx = {}
    this.each(function(item) {idx[keyFunc(item)] = item});
    return idx;
}

Array.prototype.flatmap = function(iterator) {
  var results = [];
  this.each(function(value, index) {
    results.push.apply(results, iterator(value, index));
  });
  return results;
}

List = Class.create();

List.prototype = {
    initialize: function(head, tail) {
        this.head = head;
	if (tail) {
		if (tail instanceof List) {
			this.tail = tail;
		} else {
			this.tail = new List(tail);
		}
	}
    },

    toArray: function() {
        var result = new Array();
	if (!this.isEmpty()) {
	    result.push(this.head);
	    var tail = this.tail;
	    while(!tail.isEmpty()) {
		result.push(tail.head);
		tail = tail.tail;
	    }
        }
        return result;
    },
    
    toString: function() {
	return this.isEmpty() ? "[]" : "cons(" + this.head.toString() + " . " + this.tail.toString() + ")";
    },
    
    isEmpty: function() {
	return this.head === undefined;
    }
}


function list(array) {
    var l = new List();
    for (var i = array.length - 1; i >= 0; i--) {
        l = cons(array[i], l);
    }
    return l;
}

function cons(head, tail) {
    return new List(head, tail);
}

function member(selector) {
    return function(obj) {return obj[selector]};
}

function callMember(selector) {
    var args = $A(arguments)
    args.shift();
    return function(obj) {return obj[selector].apply(obj, args)};
}

String.prototype.toCapitalized = function() {
    return this.charAt(0).toUpperCase() + this.substring(1);
}

String.prototype.escapeHtml = function() {
    return this.replace(/<|>|&|'|"/g, function(match) {return "&#" + match[0].charCodeAt(0) + ";"});
}
